@@ -1,10 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import datetime
+import operator
 from decimal import Decimal
 
-from trytond.model import (ModelView, ModelSQL, MatchMixin, ValueMixin, fields,
-    sequence_ordered)
+from trytond.model import (
+    ModelView, ModelSQL, MatchMixin, ValueMixin, DeactivableMixin, fields,
+    sequence_ordered, tree)
 from trytond import backend
 from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
@@ -21,14 +23,13 @@ STATES = {
 DEPENDS = ['active']
 
 
-class Location(ModelSQL, ModelView):
+class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
     "Stock Location"
     __name__ = 'stock.location'
     name = fields.Char("Name", size=None, required=True, states=STATES,
         depends=DEPENDS, translate=True)
     code = fields.Char("Code", size=None, states=STATES, depends=DEPENDS,
         select=True)
-    active = fields.Boolean('Active', select=True)
     address = fields.Many2One("party.address", "Address",
         states={
             'invisible': Eval('type') != 'warehouse',
@@ -108,9 +109,11 @@ class Location(ModelSQL, ModelView):
             ],
         depends=['type', 'active', 'storage_location'],
         help='If empty the Storage is used')
-    quantity = fields.Function(fields.Float('Quantity'), 'get_quantity')
-    forecast_quantity = fields.Function(fields.Float('Forecast Quantity'),
-            'get_quantity')
+    quantity = fields.Function(
+        fields.Float('Quantity'), 'get_quantity', searcher='search_quantity')
+    forecast_quantity = fields.Function(
+        fields.Float('Forecast Quantity'), 'get_quantity',
+        searcher='search_quantity')
     cost_value = fields.Function(fields.Numeric('Cost Value'),
         'get_cost_value')
 
@@ -141,7 +144,7 @@ class Location(ModelSQL, ModelView):
                 ()),
             ]
         childs_mapping = cls._childs_domain()
-        for type_, allowed_parents in cls._parent_domain().iteritems():
+        for type_, allowed_parents in cls._parent_domain().items():
             parent_domain.append(If(Eval('type') == type_,
                     ('type', 'in', allowed_parents), ()))
             childs_domain.append(If(Eval('type') == type_,
@@ -167,7 +170,7 @@ class Location(ModelSQL, ModelView):
     @classmethod
     def _childs_domain(cls):
         childs_domain = {}
-        for type_, allowed_parents in cls._parent_domain().iteritems():
+        for type_, allowed_parents in cls._parent_domain().items():
             for parent in allowed_parents:
                 childs_domain.setdefault(parent, [])
                 childs_domain[parent].append(type_)
@@ -175,16 +178,14 @@ class Location(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
         super(Location, cls).__register__(module_name)
 
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         table.index_action(['left', 'right'], 'add')
 
     @classmethod
     def validate(cls, locations):
         super(Location, cls).validate(locations)
-        cls.check_recursion(locations)
         inactives = []
         for location in locations:
             location.check_type_for_moves()
@@ -229,7 +230,7 @@ class Location(ModelSQL, ModelView):
             locations = cls.search([])
         if not locations:
             return []
-        location_ids = map(int, locations)
+        location_ids = list(map(int, locations))
         # Use root to compute for all companies
         # and ensures inactive locations are in the query
         with Transaction().set_user(0), \
@@ -239,7 +240,7 @@ class Location(ModelSQL, ModelView):
             quantities = Move.compute_quantities(
                 query, location_ids, with_childs=True)
             empty = set(location_ids)
-            for (location_id, product), quantity in quantities.iteritems():
+            for (location_id, product), quantity in quantities.items():
                 if quantity:
                     empty.discard(location_id)
             for sub_ids in grouped_slice(list(empty)):
@@ -256,10 +257,6 @@ class Location(ModelSQL, ModelView):
                         empty.discard(location.id)
         return cls.browse(empty)
 
-    @staticmethod
-    def default_active():
-        return True
-
     @staticmethod
     def default_left():
         return 0
@@ -277,7 +274,7 @@ class Location(ModelSQL, ModelView):
         return 'storage'
 
     @classmethod
-    def check_xml_record(self, records, values):
+    def check_xml_record(cls, records, values):
         return True
 
     def get_warehouse(self, name):
@@ -292,74 +289,117 @@ class Location(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        locations = cls.search([
-                ('code', '=', clause[2]),
-                ], order=[])
-        if locations:
-            return [('id', 'in', [l.id for l in locations])]
-        return [(cls._rec_name,) + tuple(clause[1:])]
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
+            (cls._rec_name,) + tuple(clause[1:]),
+            ('code',) + tuple(clause[1:]),
+            ]
 
     @classmethod
     def get_quantity(cls, locations, name):
         pool = Pool()
         Product = pool.get('product.product')
         Date_ = pool.get('ir.date')
+        trans_context = Transaction().context
 
-        if (not Transaction().context.get('product')) \
-                or not (isinstance(Transaction().context['product'],
-                    (int, long))):
-            return dict([(l.id, 0) for l in locations])
+        def valid_context(name):
+            return (trans_context.get(name) is not None
+                and isinstance(trans_context[name], int))
 
-        with Transaction().set_context(active_test=False):
-            if not Product.search([
-                        ('id', '=', Transaction().context['product']),
-                        ]):
-                return dict([(l.id, 0) for l in locations])
+        if not any(map(valid_context, ['product', 'product_template'])):
+            return {l.id: None for l in locations}
 
         context = {}
         if (name == 'quantity'
-                and Transaction().context.get('stock_date_end', datetime.date.max) >
-                Date_.today()):
+                and (trans_context.get('stock_date_end', datetime.date.max)
+                    > Date_.today())):
             context['stock_date_end'] = Date_.today()
 
         if name == 'forecast_quantity':
             context['forecast'] = True
-            if not Transaction().context.get('stock_date_end'):
+            if not trans_context.get('stock_date_end'):
                 context['stock_date_end'] = datetime.date.max
 
-        product_id = Transaction().context['product']
+        if trans_context.get('product') is not None:
+            grouping = ('product',)
+            grouping_filter = ([trans_context['product']],)
+            key = trans_context['product']
+        else:
+            grouping = ('product.template',)
+            grouping_filter = ([trans_context['product_template']],)
+            key = trans_context['product_template']
         pbl = {}
         for sub_locations in grouped_slice(locations):
             location_ids = [l.id for l in sub_locations]
             with Transaction().set_context(context):
                 pbl.update(Product.products_by_location(
-                        location_ids=location_ids, product_ids=[product_id],
-                        with_childs=True))
+                        location_ids,
+                        grouping=grouping,
+                        grouping_filter=grouping_filter,
+                        with_childs=trans_context.get('with_childs', True)))
 
-        return dict((loc.id, pbl.get((loc.id, product_id), 0))
-            for loc in locations)
+        return dict((loc.id, pbl.get((loc.id, key), 0)) for loc in locations)
+
+    @classmethod
+    def search_quantity(cls, name, domain):
+        _, operator_, operand = domain
+        operator_ = {
+            '=': operator.eq,
+            '>=': operator.ge,
+            '>': operator.gt,
+            '<=': operator.le,
+            '<': operator.lt,
+            '!=': operator.ne,
+            'in': lambda v, l: v in l,
+            'not in': lambda v, l: v not in l,
+            }.get(operator_, lambda v, l: False)
+
+        ids = []
+        for location in cls.search([]):
+            if operator_(getattr(location, name), operand):
+                ids.append(location.id)
+        return [('id', 'in', ids)]
 
     @classmethod
     def get_cost_value(cls, locations, name):
-        Product = Pool().get('product.product')
+        pool = Pool()
+        Product = pool.get('product.product')
+        Template = pool.get('product.template')
         trans_context = Transaction().context
-        product_id = trans_context.get('product')
-        if not product_id:
-            return dict((l.id, None) for l in locations)
-        cost_values, context = {}, {}
+        cost_values = {l.id: None for l in locations}
+
+        def valid_context(name):
+            return (trans_context.get(name) is not None
+                and isinstance(trans_context[name], int))
+
+        if not any(map(valid_context, ['product', 'product_template'])):
+            return cost_values
+
+        def get_record():
+            if trans_context.get('product') is not None:
+                return Product(trans_context['product'])
+            else:
+                return Template(trans_context['product_template'])
+
+        context = {}
         if 'stock_date_end' in trans_context:
             # Use the last cost_price of the day
             context['_datetime'] = datetime.datetime.combine(
                 trans_context['stock_date_end'], datetime.time.max)
+            # The date could be before the product creation
+            record = get_record()
+            if record.create_date > context['_datetime']:
+                return cost_values
         with Transaction().set_context(context):
-            product = Product(product_id)
+            cost_price = get_record().cost_price
+        # The template may have more than one product
+        if cost_price is not None:
             for location in locations:
-                # The date could be before the product creation
-                if not isinstance(product.cost_price, Decimal):
-                    cost_values[location.id] = None
-                else:
-                    cost_values[location.id] = (Decimal(str(location.quantity))
-                        * product.cost_price)
+                cost_values[location.id] = (
+                    Decimal(str(location.quantity)) * cost_price)
         return cost_values
 
     @classmethod
@@ -425,6 +465,8 @@ class Location(ModelSQL, ModelView):
     def copy(cls, locations, default=None):
         if default is None:
             default = {}
+        else:
+            default = default.copy()
 
         res = []
         for location in locations:
@@ -460,7 +502,7 @@ class Location(ModelSQL, ModelView):
                 if location.id in warehouse_locations.values():
                     cp_warehouse = cls(
                         Transaction().context['cp_warehouse_id'])
-                    for field, loc_id in warehouse_locations.iteritems():
+                    for field, loc_id in warehouse_locations.items():
                         if loc_id == location.id:
                             cls.write([cp_warehouse], {
                                     field: new_location.id,
@@ -479,8 +521,7 @@ customer_location = fields.Many2One(
     )
 
 
-class Party:
-    __metaclass__ = PoolMeta
+class Party(metaclass=PoolMeta):
     __name__ = 'party.party'
     supplier_location = fields.MultiValue(supplier_location)
     customer_location = fields.MultiValue(customer_location)
